@@ -1,4 +1,4 @@
-# $Id: genclass.pl,v 1.11 2002/06/21 17:22:34 dmlloyd Exp $
+# $Id: genclass.pl,v 1.12 2002/07/23 17:57:00 mongo Exp $
 
 use FindBin;
 use lib "$FindBin::Bin/../lib";
@@ -38,6 +38,9 @@ for (@{$vtbl}) {
     my ($retval, $methname, $args) = @{$_};
     
     print "    $retval $methname ($args) {\n";
+    if($methname eq "name") {
+        print "        return whoami\;\n";
+    }
     print "    }\n\n";
 }
 
