@@ -34,6 +34,10 @@ sub runstep {
     ($osname, $cpuarch) = ($cpuarch, "");
   }
 
+  if($osname =~ /darwin/) {
+    $cpuarch = 'ppc';
+  }
+
   if($cpuarch =~ /MSWin32/) {
     $cpuarch = 'i386';
     $osname  = 'MSWin32';
@@ -42,14 +46,17 @@ sub runstep {
     $cpuarch = 'i386';
   }
 
+  $cpuarch               =~ s/armv[34]l?/arm/i;
+
   Configure::Data->set(
     archname    => $archname,
     cpuarch     => $cpuarch,
     osname      => $osname,
   );
 
+
+  $cpuarch                     =~ s/i[456]86/i386/i;
   my $jitarchname              =  "$cpuarch-$osname";
-  $jitarchname                 =~ s/i[456]86/i386/i;
   $jitarchname                 =~ s/-(net|free|open)bsd$/-bsd/i;
   my $jitcapable               =  0;
 
