@@ -87,7 +87,7 @@ Parrot_dlerror(void)
 */
 
 void *
-Parrot_dlsym(void *handle, char *symbol)
+Parrot_dlsym(void *handle, const char *symbol)
 {
     return (void *)(ptrcast_t)GetProcAddress(handle, symbol);
 }
