@@ -1,14 +1,12 @@
-my($ccflags, $ldflags, $libs)=Configure::Data->get(qw(ccflags ldflags libs));
+my($ccflags, $ldflags)=Configure::Data->get(qw(ccflags ldflags));
 
 $ccflags .= " -pipe -fno-common ";
 $ccflags =~ s/-flat_namespace\s*//;
 $ldflags =~ s/-flat_namespace\s*//;
-$ldflags .= " -L/sw/lib -flat_namespace ";
-$libs    .= " -ldl ";
+$ldflags .= " -flat_namespace ";
 
 Configure::Data->set(
   ccflags => $ccflags,
   ldflags => $ldflags,
   cc_warn => "-Wno-shadow",
-  libs => $libs
 );
