@@ -2,7 +2,7 @@
  * disassemble.c
  *
  * CVS Info
- *    $Id: disassemble.c,v 1.2 2002/06/07 01:31:57 josh Exp $
+ *    $Id: disassemble.c,v 1.3 2002/08/17 01:11:08 sfink Exp $
  * Overview:
  *    Parrot disassembler
  * History:
@@ -33,7 +33,7 @@ main(int argc, char *argv[])
         return 1;
     }
 
-    Parrot_init(interpreter);
+    Parrot_init(interpreter, (void*) &interpreter);
 
     if (argc != 2) {
         fprintf(stderr, "Usage: disassemble programfile \n");
