@@ -1,7 +1,7 @@
 /* embed.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: embed.h,v 1.13 2002/07/15 02:52:00 brentdax Exp $
+ *     $Id: embed.h,v 1.15 2002/08/26 14:17:18 grunblatt Exp $
  *  Overview:
  *     This is the Parrot embedding system--the only part of Parrot that
  *     the outside world should see.
@@ -24,7 +24,7 @@ typedef void *Parrot_Interp_flag_val;
 
 Parrot_Interp Parrot_new(void);
 
-void Parrot_init(Parrot_Interp);
+void Parrot_init(Parrot_Interp, void*);
 
 void Parrot_setflag(Parrot_Interp, Parrot_Interp_flag, Parrot_Interp_flag_val);
 
@@ -34,11 +34,13 @@ Parrot_PackFile Parrot_readbc(Parrot_Interp, char *);
 
 void Parrot_loadbc(Parrot_Interp, Parrot_PackFile);
 
+void Parrot_setup_argv(Parrot_Interp, int argc, char ** argv);
+
 void Parrot_runcode(Parrot_Interp, int argc, char *argv[]);
 
 void Parrot_destroy(Parrot_Interp);
 
-void Parrot_debug(Parrot_Interp);
+void Parrot_debug(Parrot_Interp, int argc, char ** argv);
 
 void Parrot_disassemble(Parrot_Interp);
 
