@@ -1,7 +1,7 @@
 /* interpreter.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: interpreter.h,v 1.50 2002/07/18 04:30:42 mongo Exp $
+ *     $Id: interpreter.h,v 1.53 2002/08/23 07:55:22 mongo Exp $
  *  Overview:
  *     The interpreter api handles running the operations
  *  Data Structure and Algorithms:
@@ -89,6 +89,7 @@ typedef struct Parrot_Context {
     struct SRegChunk *string_reg_base;  /* Base of the string stack */
     struct PRegChunk *pmc_reg_base;     /* Base of the PMC stack */
 
+    struct Stack_Chunk *pad_stack;      /* Base of the lex pad stack */
     struct Stack_Chunk *user_stack;     /* Base of the scratch stack */
     struct Stack_Chunk *control_stack;  /* Base of the flow control stack */
     IntStack intstack;                  /* Base of the regex stack */
@@ -105,7 +106,6 @@ typedef struct Parrot_Interp {
 
     struct Stash *perl_stash;           /* Pointer to the global variable
                                          * area */
-    struct Scratchpad *cur_pad;         /* The current scratchpad */
     struct Arenas *arena_base;          /* Pointer to this interpreter's 
                                          * arena */
     void *piodata;                      /* interpreter's IO system */
@@ -137,8 +137,8 @@ typedef struct Parrot_Interp {
     void **prederef_code;       /* The predereferenced code */
     size_t current_line;        /* Which line we're executing in the
                                  * source */
-    void *current_file;         /* The file we're currently in */
-    void *current_package;      /* The package we're currently in */
+    String *current_file;       /* The file we're currently in */
+    String *current_package;    /* The package we're currently in */
 
     /* Some counters for the garbage collector.xs */
     size_t  dod_runs;           /* Number of times we've
@@ -151,8 +151,8 @@ typedef struct Parrot_Interp {
                                                  * system since the last
                                                  * compaction run */
     size_t  header_allocs_since_last_collect;   /* The number of header
-                                                 * allocs from the
-                                                 * system since the last
+                                                 * blocks allocated from 
+                                                 * the system since the last
                                                  * DOD run */
     size_t  active_PMCs;        /* The number of live PMCs */
     size_t  active_Buffers;     /* The number of live
