@@ -1,7 +1,7 @@
 /* op.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: op.h,v 1.16 2002/06/09 23:29:29 josh Exp $
+ *     $Id: op.h,v 1.18 2002/08/19 23:15:52 tom Exp $
  *  Overview:
  *     Header file for op functions.
  *  Data Structure and Algorithms:
@@ -30,12 +30,14 @@ typedef enum {
     PARROT_ARG_PC,
     PARROT_ARG_SC,
     PARROT_ARG_KC,
+    PARROT_ARG_KIC,
 
     PARROT_ARG_I,
     PARROT_ARG_N,
     PARROT_ARG_P,
     PARROT_ARG_S,
-    PARROT_ARG_K
+    PARROT_ARG_K,
+    PARROT_ARG_KI
 } arg_type_t;
 
 typedef enum {
@@ -76,8 +78,8 @@ typedef struct {
     const char *full_name;
     const char *func_name;
     const char *body;
-    const char jump;
-    INTVAL arg_count;           /* Includes opcode as one arg */
+    unsigned short jump;
+    short arg_count;           /* Includes opcode as one arg */
     arg_type_t types[PARROT_MAX_ARGS];
     arg_dir_t dirs[PARROT_MAX_ARGS];
 } op_info_t;
