@@ -1,7 +1,7 @@
 /* perltypes.h
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: perltypes.h,v 1.1 2002/07/18 02:41:56 educated_foo Exp $
+ *     $Id: perltypes.h,v 1.3 2002/08/18 23:58:43 mongo Exp $
  *  Overview:
  *     Header to be shared among Perl PMC classes
  *  Data Structure and Algorithms:
@@ -15,15 +15,15 @@
  * as necessary.
  */
 
-#define CHANGE_TYPE(thing, type) do { \
+#define CHANGE_TYPE(thing, type) { \
     if ((thing)->vtable == &Parrot_base_vtables[enum_class_PerlString]) { \
 	(thing)->flags &= ~(UINTVAL)PMC_is_buffer_ptr_FLAG; \
     } \
     (thing)->vtable = &Parrot_base_vtables[enum_class_ ## type]; \
     if (enum_class_ ## type == enum_class_PerlString) { \
-	(thing)->flags = PMC_is_buffer_ptr_FLAG; \
+         (thing)->flags = PMC_is_buffer_ptr_FLAG; \
     } \
-} while (0)
+}
 
 /*
  * Local variables:
