@@ -1,7 +1,7 @@
 /* interpreter.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: interpreter.c,v 1.93 2002/07/18 04:29:39 mongo Exp $
+ *     $Id: interpreter.c,v 1.98 2002/08/23 07:53:35 mongo Exp $
  *  Overview:
  *     The interpreter api handles running the operations
  *  Data Structure and Algorithms:
@@ -10,6 +10,7 @@
  *  References:
  */
 
+#include <assert.h>
 #include "parrot/parrot.h"
 #include "parrot/interp_guts.h"
 #include "parrot/oplib/core_ops.h"
@@ -42,7 +43,7 @@ check_fingerprint(struct Parrot_Interp *interpreter)
         const char *fp_data;
         INTVAL fp_len;
 
-        fp_data = PCONST(0)->string->bufstart;
+        fp_data = PCONST(0)->string->strstart;
         fp_len = PCONST(0)->string->buflen;
 
         if (strncmp(OPCODE_FINGERPRINT, fp_data, fp_len)) {
@@ -404,8 +405,6 @@ void
 runops(struct Parrot_Interp *interpreter, struct PackFile *code, size_t offset)
 {
     opcode_t *(*core) (struct Parrot_Interp *, opcode_t *);
-    void *dummy_ptr;
-    interpreter->lo_var_ptr = &dummy_ptr;
 
     interpreter->code = code;
     interpreter->resume_offset = offset;
@@ -567,14 +566,16 @@ make_interpreter(Interp_flags flags)
     interpreter->ctx.pmc_reg_base->prev = NULL;
     Parrot_clear_p(interpreter);
 
-    interpreter->DOD_block_level--;
-    interpreter->GC_block_level--;
+    /* Stack for lexical pads */
+    interpreter->ctx.pad_stack = new_stack(interpreter);
 
     /* Need a user stack */
     interpreter->ctx.user_stack = new_stack(interpreter);
+    assert(interpreter->ctx.user_stack->buffer != interpreter->ctx.pad_stack->buffer);
 
     /* And a control stack */
     interpreter->ctx.control_stack = new_stack(interpreter);
+    assert(interpreter->ctx.control_stack->buffer != interpreter->ctx.user_stack->buffer);
 
     /* A regex stack would be nice too. */
     interpreter->ctx.intstack = intstack_new(interpreter);
@@ -585,12 +586,6 @@ make_interpreter(Interp_flags flags)
     interpreter->op_func_table = interpreter->op_lib->op_func_table;
     interpreter->op_info_table = interpreter->op_lib->op_info_table;
 
-    /* Set I/O data to NULL first or else PIO_init will
-     * assume this interpreter is already initialized.
-     */
-    interpreter->piodata = NULL;
-    PIO_init(interpreter);
-
     /* Set up defaults for line/package/file */
     interpreter->current_line = 0;
     interpreter->current_file =
@@ -598,6 +593,18 @@ make_interpreter(Interp_flags flags)
     interpreter->current_package =
         string_make(interpreter, "(unknown package)", 18, NULL, 0, NULL);;
 
+    /* Okay, we've finished doing anything that might trigger GC. (The
+     * top of the stack hasn't been set yet, so we cannot allow GC to
+     * run up until now -- we might miss system stack values. */
+    interpreter->DOD_block_level--;
+    interpreter->GC_block_level--;
+
+    /* Set I/O data to NULL first or else PIO_init will
+     * assume this interpreter is already initialized.
+     */
+    interpreter->piodata = NULL;
+    PIO_init(interpreter);
+
     /* Done. Return and be done with it */
 
     interpreter->code = (struct PackFile *)NULL;
