@@ -3,9 +3,20 @@
 **
 ** SPARC
 **
-** $Id: jit_emit.h,v 1.3 2002/07/04 22:40:59 mrjoltcola Exp $
+** $Id: jit_emit.h,v 1.12 2002/08/13 18:48:04 grunblatt Exp $
 **/
 
+/* 
+ * SPARC JIT overview:
+ * 
+ * The interpreter pointer is kept in i0.
+ * The address of register I0 is stored in i1, with all parrot register access
+ * performed relative to this register.
+ * The address of the opcode - native code mapping array is kept in i3.
+ *
+ * See IMPORTANT SHORTCUTS below.
+ */
+
 /* Sparc register numbers */
 #define emitm_g(n) (n)
 #define emitm_o(n) ((n) + 8)
@@ -13,6 +24,8 @@
 #define emitm_i(n) ((n) + 24)
 #define emitm_f(n) (n)
 
+#if JIT_EMIT
+
 #define emitm_FP emitm_i(6)
 #define emitm_SP emitm_o(6)
 
@@ -264,17 +277,31 @@ void main(){
 /* Fixup types */
 enum  {JIT_BRANCH, JIT_CALL30 };
 
-/* Shortcuts for registers */
+/* 
+ *
+ * IMPORTANT SHORTCUTS
+ *
+ * */ 
+    
+/* The register holding the interpreter pointer */
 #define Parrot_jit_intrp emitm_i(0)
+
+/* The register holding the address of I0 */
 #define Parrot_jit_regbase emitm_i(1)
+
+/* The register containing the address of the opmap */
 #define Parrot_jit_opmap emitm_i(3)
+
+/* The scratch register used for certain address calculations */
 #define Parrot_jit_tmp emitm_l(7)
 
 #define Parrot_jit_regbase_ptr(i) &((i)->ctx.int_reg.registers[0])
+
+/* The offset of a Parrot register from the base register */
 #define Parrot_jit_regoff(a, i) (unsigned)(a) - (unsigned)(Parrot_jit_regbase_ptr(i))
 
 /* Generate conditional branch to offset from current parrot op */
-static void Parrot_jit_bicc(Parrot_jit_info *jit_info, int cond, int annul,
+static void Parrot_jit_bicc(Parrot_jit_info_t *jit_info, int cond, int annul,
                             opcode_t disp)
 {
     int offset;
@@ -282,8 +309,8 @@ static void Parrot_jit_bicc(Parrot_jit_info *jit_info, int cond, int annul,
 
     opcode = jit_info->op_i + disp;
     if(opcode <= jit_info->op_i){
-        offset = jit_info->op_map[opcode].offset -
-                    (jit_info->native_ptr - jit_info->arena_start);
+        offset = jit_info->arena.op_map[opcode].offset -
+                    (jit_info->native_ptr - jit_info->arena.start);
 
         if((offset > emitm_branch_max) || (offset < emitm_branch_min))
             internal_exception(JIT_ERROR,
@@ -294,12 +321,13 @@ static void Parrot_jit_bicc(Parrot_jit_info *jit_info, int cond, int annul,
     }
 
     Parrot_jit_newfixup(jit_info);
-    jit_info->fixups->type = JIT_BRANCH;
-    jit_info->fixups->param.opcode = opcode;
+    jit_info->arena.fixups->type = JIT_BRANCH;
+    jit_info->arena.fixups->param.opcode = opcode;
     emitm_bicc(jit_info->native_ptr, annul, cond, 0);
 }
 
-static void Parrot_jit_int_load(Parrot_jit_info *jit_info,
+/* This function loads a value */
+static void Parrot_jit_int_load(Parrot_jit_info_t *jit_info,
                              struct Parrot_Interp *interpreter,
                              int param,
                              int hwreg)
@@ -351,7 +379,7 @@ static void Parrot_jit_int_load(Parrot_jit_info *jit_info,
     }
 }
 
-static void Parrot_jit_int_store(Parrot_jit_info *jit_info,
+static void Parrot_jit_int_store(Parrot_jit_info_t *jit_info,
                              struct Parrot_Interp *interpreter,
                              int param,
                              int hwreg)
@@ -381,7 +409,7 @@ static void Parrot_jit_int_store(Parrot_jit_info *jit_info,
     }
 }
 
-static void Parrot_jit_float_load(Parrot_jit_info *jit_info,
+static void Parrot_jit_float_load(Parrot_jit_info_t *jit_info,
                              struct Parrot_Interp *interpreter,
                              int param,
                              int hwreg)
@@ -411,7 +439,7 @@ static void Parrot_jit_float_load(Parrot_jit_info *jit_info,
     }
 }
 
-static void Parrot_jit_float_store(Parrot_jit_info *jit_info,
+static void Parrot_jit_float_store(Parrot_jit_info_t *jit_info,
                              struct Parrot_Interp *interpreter,
                              int param,
                              int hwreg)
@@ -442,28 +470,28 @@ static void Parrot_jit_float_store(Parrot_jit_info *jit_info,
 }
 
 
-void Parrot_jit_dofixup(Parrot_jit_info *jit_info,
+void Parrot_jit_dofixup(Parrot_jit_info_t *jit_info,
                         struct Parrot_Interp * interpreter)
 {
-    Parrot_jit_fixup *fixup;
-    Parrot_jit_fixup *last_fixup;
+    Parrot_jit_fixup_t *fixup;
+    Parrot_jit_fixup_t *last_fixup;
     char *fixup_ptr;
     int fixup_val;
 
-    fixup = jit_info->fixups;
+    fixup = jit_info->arena.fixups;
 
     while(fixup){
         switch(fixup->type){
         /* This fixes-up a branch to a known opcode offset */
             case JIT_BRANCH:
                 fixup_ptr = Parrot_jit_fixup_target(jit_info, fixup);
-                fixup_val = (jit_info->op_map[fixup->param.opcode].offset
+                fixup_val = (jit_info->arena.op_map[fixup->param.opcode].offset
                                 - fixup->native_offset) / 4;
                 *(int *)(fixup_ptr) |= emitm_mask(22, fixup_val);
                 break;
 
             case JIT_CALL30:
-                fixup_ptr = jit_info->arena_start + fixup->native_offset;
+                fixup_ptr = jit_info->arena.start + fixup->native_offset;
                 fixup_val = (int)fixup->param.fptr - (int)fixup_ptr;
                 emitm_call_30(fixup_ptr, emitm_hi30(fixup_val));
                 break;
@@ -477,7 +505,7 @@ void Parrot_jit_dofixup(Parrot_jit_info *jit_info,
     }
 }
 
-void Parrot_jit_begin(Parrot_jit_info *jit_info,
+void Parrot_jit_begin(Parrot_jit_info_t *jit_info,
                       struct Parrot_Interp * interpreter)
 {
     int ireg0_offset;
@@ -501,12 +529,12 @@ void Parrot_jit_begin(Parrot_jit_info *jit_info,
 
     /* Setup the pointer to the opcode map */
     emitm_sethi(jit_info->native_ptr,
-        emitm_hi22(jit_info->op_map), Parrot_jit_opmap);
+        emitm_hi22(jit_info->arena.op_map), Parrot_jit_opmap);
     emitm_or_i(jit_info->native_ptr,
-        emitm_i(3), emitm_lo10(jit_info->op_map), Parrot_jit_opmap);
+        emitm_i(3), emitm_lo10(jit_info->arena.op_map), Parrot_jit_opmap);
 }
 
-void Parrot_jit_normal_op(Parrot_jit_info *jit_info,
+void Parrot_jit_normal_op(Parrot_jit_info_t *jit_info,
                           struct Parrot_Interp * interpreter)
 {
     emitm_sethi(jit_info->native_ptr, emitm_hi22(jit_info->cur_op), emitm_o(0));
@@ -514,15 +542,15 @@ void Parrot_jit_normal_op(Parrot_jit_info *jit_info,
         emitm_o(0), emitm_lo10(jit_info->cur_op), emitm_o(0));
 
     Parrot_jit_newfixup(jit_info);
-    jit_info->fixups->type = JIT_CALL30;
-    jit_info->fixups->param.fptr =
+    jit_info->arena.fixups->type = JIT_CALL30;
+    jit_info->arena.fixups->param.fptr =
         (void (*)(void))interpreter->op_func_table[*(jit_info->cur_op)];
 
     emitm_call_30(jit_info->native_ptr, 0);
     emitm_mov(jit_info->native_ptr, Parrot_jit_intrp, emitm_o(1));
 }
 
-void Parrot_jit_cpcf_op(Parrot_jit_info *jit_info,
+void Parrot_jit_cpcf_op(Parrot_jit_info_t *jit_info,
                         struct Parrot_Interp * interpreter)
 {
     Parrot_jit_normal_op(jit_info, interpreter);
@@ -543,6 +571,30 @@ void Parrot_jit_cpcf_op(Parrot_jit_info *jit_info,
     emitm_nop(jit_info->native_ptr);
 }
 
+void
+Parrot_jit_load_registers(Parrot_jit_info_t *jit_info,
+    struct Parrot_Interp *interpreter)
+{
+/* TODO write me */
+}
+
+void
+Parrot_jit_save_registers(Parrot_jit_info_t *jit_info,
+    struct Parrot_Interp * interpreter)
+{
+/* TODO write me */
+}
+
+#else
+
+#  define REQUIRES_CONSTANT_POOL 0
+#  define INT_REGISTERS_TO_MAP 1
+#  define FLOAT_REGISTERS_TO_MAP 0
+
+char intval_map[INT_REGISTERS_TO_MAP] = { emitm_l(0) }; 
+
+#endif
+
 /*
  * Local variables:
  * c-indentation-style: bsd
