@@ -17,6 +17,12 @@ grammar to the file $name.pm.  The resulting parser can then be loaded
 as a module, and instantiated with its B<new> method.  This is much
 faster than re-building the grammar.
 
+=item B<Debug($on)>
+
+If C<$on> is true, generate line-number information.  This actually
+has a surprisingly small impact on performance.  If C<$on> is false,
+turn line-number generation off.
+
 =back
 
 It's been tweaked for speed in a number of ways.  First, the infix
@@ -28,8 +34,11 @@ involved.
 
 =cut
 
+use FindBin;
+use lib "$FindBin::Bin/../../lib";
 use Parse::RecDescent;
 use strict;
+require Exporter;
 
 =head2 Internals
 
@@ -52,10 +61,9 @@ recognized.  The hash values are currently unimportant.
 
 =cut
 
-use vars qw(%WANT %CLASSES $FUNCTION_ARGS $grammar);
+use vars qw(%WANT %CLASSES $FUNCTION_ARGS $grammar $err_handler);
 
-%WANT = ();
-%CLASSES = ();
+$err_handler = undef;
 
 sub Precompile {
     shift;
@@ -98,11 +106,32 @@ sub add_class {		# seen class.
     1;
 }
 
+=item B<set_error_handler($err_handler)>
+
+Sets the error handler for the parser.
+When an error is detected during the parse,
+$err_handler is called with these parameters:
+    &$err_handler($msg, $linenum)
+
+If no error handler if set then a message is printed
+on STDERR.
+
+This function returns the previous error handler.
+
+=cut
+
+sub Parse::RecDescent::set_error_handler {
+    my $self = shift;
+    my $old_handler = $err_handler;
+    $err_handler = shift;
+    $old_handler;
+}
+
 ##############################
 # Functions (list operators):
 
 INIT {
-$FUNCTION_ARGS = 'maybe_comma';
+$FUNCTION_ARGS = 'bare_arglist';
 
 # XXX: many of these need their own special want_* rules.  This is
 # just a hack to make the parser swallow the examples from the
@@ -123,8 +152,9 @@ my @builtin_funcs = qw(crypt index pack rindex sprintf substr
 
 ##############################
 # Loop control
-my @loop_control = qw(redo last next continue);
+my @loop_control = qw(redo last next continue break skip);
 @WANT{@loop_control} = ('maybe_label') x @loop_control;
+$WANT{goto} = 'label';
 
 ##############################
 # Unary operators
@@ -132,11 +162,12 @@ my @loop_control = qw(redo last next continue);
 my @unary_ops = qw(chop chomp chr hex lc lcfirst length
 		   ord reverse uc ucfirst
 		   abs atan2 cos exp hex int log oct rand sin sqrt srand
-		   pop shift
+		   pop shift exit
 		   delete each exists keys values
 		   defined undef
 		   chdir chroot glob mkdir rmdir stat umask
-		   close);
+		   close
+		   print1);	# temporary cruft.
 @WANT{@unary_ops} = ('prefix') x @unary_ops;
 
 ##############################
@@ -153,7 +184,7 @@ $WANT{until} = 'want_for_while';
 ##############################
 # Named blocks
 
-# XXX: Will not compile.
+# XXX: Most will not compile.
 my @special_blocks = qw(CATCH BEGIN END INIT AUTOLOAD
 			PRE POST NEXT LAST FIRST
 			try do);
@@ -169,7 +200,7 @@ my @builtin_types = qw(int num str HASH ARRAY SCALAR Inf NaN
 
 BEGIN {
 # Handle comments:
-    $Parse::RecDescent::skip = '(\s*(#[^\n]*?(\n|\Z))?)+';
+    $Parse::RecDescent::skip = '(?:\s*(#[^\n]*?(\n|\Z))?)+';
     $::RD_AUTOACTION = q { bless [@item], 'P6C::'.$item[0] };
 }
 
@@ -179,15 +210,18 @@ $grammar = <<'ENDSUPPORT';
 # Support functions and variables
 {
     use vars '$no_comma';
+    use vars '$err_handler';
     use vars qw(%KEYWORDS %CLASSES %WANT);
     use vars qw($NAMEPART $COMPARE $CONTEXT $MULDIV $PREFIX $ADDSUB $INCR
 		$LOG_OR $LOGOR $FILETEST $ASSIGN $HYPE $MATCH $BITSHIFT
-		$SOB);
+		$SOB $FLUSH $NUMPART $RXATOM $RXMETA $RXCHARCLASS $RXODELIM
+		$RXESCAPED $HEXCHAR $RXASSERTION);
+    use vars '%CDELIM';
 
 # Things from P6C::* used during the parse:
 BEGIN {
     no strict 'refs';
-    for (qw(add_class add_function CLASSES WANT)) {
+    for (qw(add_class add_function CLASSES WANT err_handler)) {
 	*$_ = *{'P6C::Parser::'.$_};
     }
 }
@@ -197,27 +231,27 @@ BEGIN {
     $SOB	= qr|$Parse::RecDescent::skip(?<![^\n\s]){|o;
     $HYPE	= qr/\^?/;
     $NAMEPART	= qr/[a-zA-Z_][\w_]*/;
-    $COMPARE	= qr{cmp | eq | [gnl]e | [gl]t
-		     | <=> | [<>=!]= | < | > }x;
-    $CONTEXT	= qr{[\%\@\$\&*_?] | \+(?!\+)}x;
-    $MULDIV	= qr{[\%*x] | /(?!/)}x;
+    $COMPARE	= qr{(?:cmp|eq|[gnl]e|[gl]t)\b|<=>|[<>=!]=|<|>};
+    $CONTEXT	= qr{[\%\@\$\&*_?]|\+(?!\+)};
+    $MULDIV	= qr{[\%*x]|/(?!/)};
     $MATCH	= qr{[=!]~};
-    $INCR	= qr{\+\+ | --}x;
-    $PREFIX	= qr{[!~\\] | -(?![->])}x;
+    $INCR	= qr{\+\+|--};
+    $PREFIX	= qr{[!~\\]|-(?![->])};
     $ADDSUB	= qr{[-+_]};
-    $BITSHIFT	= qr{<< | >>}x;
-    $LOG_OR	= qr{x?or | err}x;
-    $LOGOR	= qr{\|\| | ~~ | //}x;
-    $FILETEST	= qr{-[rwxoRWXOezsfdlpSbctugkTBMAC]+};
-    $ASSIGN	= qr{(?:
-		     ! | :	# != and :=
-		     | //
-		     | &&? | \|\|? | ~~?
-		     | << | >>
-		     | $ADDSUB
-		     | $MULDIV
-		     | \*\*
-		    )? =}x;
+    $BITSHIFT	= qr{<<|>>};
+    $LOG_OR	= qr{(?:x?or|err)\b};
+    $LOGOR	= qr{\|\||~~|//};
+    $FILETEST	= qr{-[rwxoRWXOezsfdlpSbctugkTBMAC]+\b};
+    $ASSIGN	= qr{(?:!|:|//|&&?|\|\|?|~~?|<<|>>|$ADDSUB|$MULDIV|\*\*)?=};
+    # Used for flushing syntax errors
+    $FLUSH	= qr/\w+|[^\s\w;}#'"]+/;
+    $NUMPART	= qr/(?!_)[\d_]+(?<!_)/;
+    $HEXCHAR	= qr/[a-fA-F0-9]{2,4}/;
+    $RXESCAPED	= qr/\\(?:[Xx]$HEXCHAR|0[0-7]{1,3}|[Xx]\{$HEXCHAR\}|[Pp]\{\w+\}|.)/o;
+    $RXASSERTION= qr/:{1,3}|\^{1,2}|\${1,2}/;
+    $RXATOM	= qr/(?:[\w_]|\\.)+/;
+    $RXCHARCLASS= qr/\[(?:[^\]\\]|\\.)+\]/;
+    $RXODELIM	= qr/[\[{(<\/!?=>)}\]#]|\s+\S/;
 }
 
 # HACK to distinguish between "my ($a, $b) ..." and "foo ($a, $b)".
@@ -228,6 +262,12 @@ BEGIN {
     @KEYWORDS{@kw} = (1) x @kw;
 }
 
+# Matching delimiters:
+BEGIN {
+    %CDELIM = qw|( )  { }  [ ]  < >
+		 ) (  } {  ] [  > <|;
+}
+
 # (see Parse::RecDescent::Consumer)
 sub consumer {
     my $t = shift;
@@ -259,6 +299,65 @@ sub mark_end {
     $since{$type} = consumer($text);
 }
 
+my $seen_err;
+
+sub Debug {
+    my ($parser, $on) = @_;
+    if ($on) {
+	$parser->Replace("\nstmts:            __stmt[\$arg[0]](s?)\n");
+    } else {
+	$parser->Replace("\nstmts:		  _stmt[\$arg[0]](s?)\n");
+    }
+}
+
+sub getl($) {
+    my $linenum = shift;
+    ref($linenum)? Parse::RecDescent::linenum($linenum) : $linenum;
+}
+
+sub gett($) {
+    my $t = shift;
+    $t =~ s/\A($Parse::RecDescent::skip)//o;
+    my $matched = $1;
+    $::line += $matched =~ s/\n//g;
+    $t =~ s/\n.*//s;
+    $t =~ s/\s*$//;
+    return length($t)> 30 ? substr($t,0,30). '...':$t;
+}
+
+sub got_err {
+    my ($msg, $text, $linenum) = @_;
+
+    $seen_err = 1;
+
+    # Take care of the hacked P::RD::linenum, if needed.
+    $linenum = getl($linenum);
+
+    # Remove whitespace at start of text.
+    $text =~ s/\A($Parse::RecDescent::skip)//o;
+
+    # Adjust line number to take account of removed whitespace.
+    my $matched = $1;
+    $linenum += $matched =~ s/\n//g;
+
+    # Remove everything after \n
+    $text =~ s/\n.*$//s;
+
+    # Make text more readable
+    if ($text eq "") {
+        $text = "end of input";
+    } else {
+        $text = qq{"$text"};
+    }
+    $msg = "$msg near $text";
+
+    if ($err_handler) {
+        &$err_handler($msg, $linenum);
+    } else {
+        print STDERR "Parse error [line $linenum]: $msg\n";
+    }
+}
+
 }
 
 ENDSUPPORT
@@ -267,7 +366,7 @@ $grammar .= <<'ENDGRAMMAR';
 ##############################
 # Literals:
 
-sv_literal:	  /(?:\d+(?:\.\d+)?|\.\d+)(?:[Ee]-?\d+)?/
+sv_literal:	  /(?:$NUMPART(?:\.$NUMPART)?|\.$NUMPART)(?:[Ee]-?$NUMPART)?/o
 		| '{' <commit> hv_seq '}'
 		| '[' <commit> av_seq(?) ']'
 		| <perl_quotelike>
@@ -279,6 +378,8 @@ hv_seq:		  <leftop: pair ',' pair> /,?/
 ##############################
 # Variables:
 variable:	  sigil <skip:''> varname
+scalar_var:	  '$' <skip:''> varname
+nonscalar_var:	  /[\@\%]/ <skip:''> varname
 
 sigil:		  /[\@\%\$\&]/
 
@@ -288,7 +389,7 @@ varname:	  name
 		| '^' <commit> namepart
 		| ('*')(?) '{' <skip:'\s*'> (scalar_expr | name) '}'
 
-name:		  /(?:::|\.|\*)?$NAMEPART(::$NAMEPART)*/o
+name:		  /(?:::|\.|\*)?$NAMEPART(?:::$NAMEPART)*/o
 
 namepart:	  /$NAMEPART/o
 
@@ -322,6 +423,7 @@ context:	  /$CONTEXT/o
 term:		  '<' <commit> expr(?) '>'
 		| subscriptable <commit> subscript(s?)
 		| /$CONTEXT/o <commit> term
+		| pattern
 		| sv_literal
 		| class
 		| closure
@@ -330,14 +432,14 @@ apply_rhs:	  namepart <commit> subscript(s?)
 		| subscript(s)
 
 apply:		  <leftop: term apply_op apply_rhs>
-apply_op:	  /$HYPE \./xo
+apply_op:	  /$HYPE\./o
 
 incr:		  incr_op <commit> apply
 		| apply incr_op(?)
-incr_op:	  /$HYPE $INCR/xo
+incr_op:	  /$HYPE$INCR/o
 
 pow:		  <leftop: incr pow_op prefix>
-pow_op:		  /$HYPE \*\*/ox
+pow_op:		  /$HYPE\*\*/o
 
 prefix:		  filetest_op <commit> prefix
 		| prefix_op <commit> prefix
@@ -345,7 +447,7 @@ prefix:		  filetest_op <commit> prefix
 		| pow
 
 # prefix_op:	  '!' | '~' | '\\' | /-(?![->])/
-prefix_op:	  /$HYPE $PREFIX/ox
+prefix_op:	  /$HYPE$PREFIX/o
 filetest_op:	  /$FILETEST/o
 
 pair:		  namepart '=>' <commit> prefix
@@ -354,36 +456,36 @@ maybe_pair:	  namepart '=>' <commit> prefix
 		| prefix ('=>' prefix)(?)
 
 match:		  <leftop: maybe_pair match_op maybe_pair>
-match_op:	  /$HYPE $MATCH/xo
+match_op:	  /$HYPE$MATCH/o
 
 muldiv:		  <leftop: match muldiv_op match>
 # muldiv_op:	  '*' | '/' | '%' | 'x'
-muldiv_op:	  /$HYPE $MULDIV/ox
+muldiv_op:	  /$HYPE$MULDIV/o
 
 addsub:		  <leftop: muldiv addsub_op muldiv>
 # addsub_op:	  '+' | '-' | '_'
-addsub_op:	  /$HYPE $ADDSUB/ox
+addsub_op:	  /$HYPE$ADDSUB/o
 
 bitshift:	  <leftop: addsub bitshift_op addsub>
-bitshift_op:	  /$HYPE $BITSHIFT/xo
+bitshift_op:	  /$HYPE$BITSHIFT/o
 
 compare:	  <leftop: bitshift compare_op bitshift>
-compare_op:	  /$HYPE $COMPARE/ox
+compare_op:	  /$HYPE$COMPARE/o
 # compare_op:	  '<=>' | '<=' | '==' | '>=' | '<' | '>' | '!='
 # 		| 'eq' | 'ge' | 'ne' | 'le' | 'lt' | 'gt' | 'cmp'
 
 bitand:		  <leftop: compare bitand_op compare>
-bitand_op:	  /$HYPE &(?!&)/ox
+bitand_op:	  /$HYPE&(?!&)/o
 
 bitor:		  <leftop: bitand bitor_op bitand>
-bitor_op:	  /$HYPE ([|~])(?!\1)/ox
+bitor_op:	  /$HYPE([|~])(?!\1)/o
 
 logand:		  <leftop: bitor logand_op bitor>
-logand_op:	  /$HYPE &&/ox
+logand_op:	  /$HYPE&&/o
 
 logor:		  <leftop: logand logor_op logand>
 # logor_op:	  '||' | '~~' | '//'
-logor_op:	  /$HYPE $LOGOR/ox
+logor_op:	  /$HYPE$LOGOR/o
 
 range:		  logor (range_op logor)(?)
 range_op:	  '..'
@@ -413,7 +515,7 @@ assign_lhs:	  scope_class decl
 		| ternary
 assign_rhs:	  assign_op scalar_expr
 
-assign_op:	  /$HYPE $ASSIGN/ox
+assign_op:	  /$HYPE$ASSIGN/o
 # assign_op:	  /[!:]?=/ <commit>
 # 		| assignable_op <skip:''> '='
 # assignable_op:	  '//'
@@ -427,8 +529,9 @@ but:		  'but' assign
 comma:		  <leftop: <matchrule:@{[@arg ? $arg[0] : 'scalar_expr']}>
 			comma_op <matchrule:@{[@arg?$arg[0]:'scalar_expr']}> >
 comma_op:	  ','
-maybe_comma:	  ...!'(' comma[@{[@arg ? $arg[0] : ()]}]
-		| ...!'('
+maybe_comma:	  comma[@{[@arg ? $arg[0] : ()]}]
+		| '' # Nothing.
+bare_arglist:	  ...!'(' maybe_comma
 
 semi:		  <leftop: expr semi_op expr>
 semi_op:	  ';'
@@ -437,11 +540,11 @@ adverb:		  comma adv_clause(?)
 adv_clause:	  /:(?!:)/ comma['scalar_expr']
 
 log_AND:	  <leftop: adverb log_AND_op adverb>
-log_AND_op:	  /$HYPE and/xo
+log_AND_op:	  /${HYPE}and\b/o
 
 log_OR:		  <leftop: log_AND log_OR_op log_AND>
 # log_OR_op:	  'or' | 'xor' | 'err'
-log_OR_op:	  /$HYPE $LOG_OR/ox
+log_OR_op:	  /$HYPE$LOG_OR/o
 
 expr:		  log_OR
 
@@ -470,35 +573,88 @@ initializer:	  assign_op scalar_expr
 ##############################
 # Statements
 
-prog:		  /\A/ stmts /\z/
-		| <error>
+prog:		  {%since=(); $seen_err=undef; $::file = $arg[0] || ''} <reject>
+		| /\A/ stmts[0] /\z/ <commit> {$seen_err ? undef : 1} ''
+		| {got_err("Invalid statement", $text, $thisline); undef;}
 
-stmts:	  	  <leftop: stmt stmt_sep stmt> stmt_sep(?)
-		| # nothing
+# this rule get's substiuted by -g switch in perl6
+# so that __stmt is used.
+stmts:		  _stmt[$arg[0]](s?)
+
+# collect file,line info
+__stmt:		{ $::line = getl($thisline);
+                  $::txt = gett($text) } <reject>
+
+__stmt:		  _stmt[$arg[0]] {
+		    bless( [$::file,$::line, '??',$item[1], $::txt],
+		    	'P6C::debug_info') }
+
+# $arg[0] is set to true/false. True == "in block"
+_stmt:		# Handle valid reason for 'stmt' failing (hit end of block)
+		  { $arg[0] ? 1 : undef } ..."}" <commit> { undef }
+
+		# Handle case where having missing input
+		# or have hit end of file.
+		| /\z/
+		  <commit>
+		  { got_err("Missing closing brace.", $text, $thisline) if ($arg[0]) }
+		  <reject>
+
+		# Handle "normal" case.
+		| stmt stmt_sep[$arg[0]] { $item[1] }
+
+		# Handle case where 'stmt' failed - try to remove
+		# bad statement. Allow this to succeed so that
+		# another statement can be tried if the flush
+		# didn't hit the end of input.
+		| { got_err("Invalid statement", $text, $thisline); 1; }
+		  _stmt_flush
+		  {0}
+
+# Heuristic for removing invalid statement:
+# Skip over all tokens up until we see a ';' or '}'
+# or hit the end of the input.
+# The tricky part is to ignore ';' or '}' that are
+# embedded in quoted things or comments.
+# The $FLUSH matches a word (\w+) or
+# one or more non-word chars (\W) except these chars:
+#     ; }   - we are looking for these
+#     ' "   - handled by perl_quotelike
+#     \s    - handled by skip
+#     #     - handled by skip
+# The _stmt_flush rule always succeeds.
+# XXX - a smarter heuristic would match { with }
+_stmt_flush:	  _stmt_token(s?) /[;}]|\z/ {0}
+
+_stmt_token:	  <perl_quotelike>
+		| /$FLUSH/o
 
 stmt_sep:	  ';'
 		| { check_end('block', $text) }
 		| { check_end('label', $text) }
+		| { $arg[0] ? 1 : undef } <commit> ..."}"
+		| { $arg[0] ? undef : 1 } .../\z/
 
 stmt:		  label /:(?!:)/ { mark_end('label', $text);1 } ''
 		| directive <commit> name comma(?)
-		| 'method' <commit> name params props['is'] block
-		| 'loop' <commit>
-			'(' scalar_expr(?)
-			';' scalar_expr(?)
-			';' scalar_expr(?) ')'
+		| /method\b/ <commit> name params props['is'] block
+		| /loop\b/ <commit>
+			'(' comma(?)
+			';' comma(?)
+			';' comma(?) ')'
 			block
-		| scope(?) 'sub' <commit> name params
+		| scope(?) /(?:sub|rule)\b/ <commit> name params
 			{ add_function(@item[4,5], $thisparser);1 }
-			props['is'] (';' | block)
+			props['is']
+		(';' | <matchrule:@{[$arg[0] eq 'sub'?'block':'rule']}>)[$item[2]]
 		| scope(?) 'class' <commit> name { add_class($item[4]);1 }
 			props['is'] block
 		| expr guard(?)
 		| ...';'
 
-directive:	  /package|module|use/
+directive:	  /(?:package|module|use)\b/
 guard:		  { check_end('block', $text) ? undef : 1 } _guard
-_guard:		  /if|unless|while|until/ <commit> scalar_expr
+_guard:		  /(?:if|unless|while|until)\b/ <commit> scalar_expr
 		| 'for' expr
 
 maybe_label:	  label(?)
@@ -507,13 +663,15 @@ label:		  namepart { exists($KEYWORDS{$item[1][1]}) ? undef
 
 block:		  start_block '...' <commit> '}'
 			{ mark_end('block', $text);1; } ''
-		| start_block stmts '}'
+		| start_block stmts[1] '}'
 			{ mark_end('block', $text);1; } ''
 
 start_block:	  <skip:''> /$SOB/o
 
-closure:	  '->' '(' <commit> _closure_args(?) ')' block
-		| '->' <commit> _closure_args(?) block
+closure:	  '->' '(' <commit> _closure_args(?) ')'
+			<matchrule:@{[$item[1] eq 'rule'?'rule':'block']}>
+		| '->' <commit> _closure_args(?)
+			<matchrule:@{[$item[1] eq 'rule'?'rule':'block']}>
 		| block
 
 _closure_args:	  <leftop: comma['variable'] semi_op comma['variable']>
@@ -536,6 +694,76 @@ want_for_sort:	  scalar_expr comma
 maybe_decl:	  scope_class <commit> variable props['is']
 		| variable <commit> props['is']
 		| # nada
+nothing:	  ''
+no_args:	  ...!'('
+
+##############################
+# Regexes:
+
+rule:	 	  rx_mod(s?) start_block rx_alt '}'
+			{ mark_end('block', $text);1; } ''
+
+pattern:	  <rulevar:$cdelim>
+pattern:	  /(?:m|qr)\b/ <commit> rx_mod(s?) <skip:''> /$RXODELIM/o
+			<skip:$item[-2]> rx_alt
+			{ $item[-2] =~ s/^\s*//;
+			  $cdelim = $CDELIM{$item[-2]} || $item[-2] } "$cdelim"
+		| '/' rx_alt '/'
+
+rx_alt:		  <leftop: rx_seq '|' rx_seq>
+
+rx_seq:		  rx_maybe_hypo(s?)
+
+rx_maybe_hypo:	  scalar_var    ':=' <commit> rx_element
+		| nonscalar_var ':=' <commit> '[' rx_alt ']' rx_repeat(?)
+		| rx_element
+		| /$RXASSERTION/o
+
+rx_element:	  ...'{' <commit> block
+		| rx_atom rx_repeat(?)
+
+rx_atom:	  '(' <commit> rx_alt ')'
+		| '[' <commit> rx_alt ']'
+		| ...':' <commit> rx_mod
+		| '<' <commit> /!?/ rx_assertion '>'
+		| '.'
+		| /$RXESCAPED/o
+		| variable
+		| /$RXATOM/o
+
+rx_mod:		  /:[\w_]+/ ( '(' maybe_comma ')' )(?)
+
+rx_repeat:	  _rx_repeat /\??/
+
+_rx_repeat:	  '<' <commit> /!?/ _rx_repspec '>'
+		| /[*+?]/
+
+_rx_repitem:	  scalar_var
+		| /\d+/
+
+_rx_repspec:	  _rx_repitem ',' _rx_repitem
+		| /\d+/
+
+rx_assertion:	  variable
+		| '.'
+		| /'(?:[^']|\\.)*'/
+		| '(' <commit> expr ')'
+		| '{' <commit> expr '}'
+		| rx_call
+		| rx_charclass
+
+rx_charclass:	  <leftop: rx_cc_neg /[+\-]/ rx_cc_neg>
+rx_cc_neg:	  /-?/ rx_cc_atom
+rx_cc_atom:	  /$RXCHARCLASS/o
+		| '<' rx_assertion '>'
+		| namepart
+
+rx_call:	  rx_rulename '(' <commit> maybe_comma ')'
+		| rx_rulename ':' <commit> /(?:[^>]|\\.)*/
+		| rx_rulename rx_alt(?)
+
+rx_rulename:	  name
+		| scalar_var
 
 ENDGRAMMAR
 
@@ -590,14 +818,20 @@ sub optional_last($) {
 # Build an argument context from a parameter spec.
 sub argument_context {
     my ($name, $params, $parser) = @_;
-    if ($params->min == 0 && !defined($params->max)) {
-	# List operator.
-	$WANT{$name} = $FUNCTION_ARGS;
-	return 1;
+    if ($params->min == 0) {
+	if (!defined($params->max)) {
+	    # List operator.
+	    $WANT{$name} = $FUNCTION_ARGS;
+	    return 1;
+	} elsif ($params->max == 0) {
+	    # no-arg sub
+	    $WANT{$name} = 'no_args';
+	    return 1;
+	}
     }
-    my $rule;
+    my $rule = "...!'(' ";
     my $rulename = "want_for_$name";
-    my $lastparam = 1;
+    my $lastparam = 2;		# to skip ...!'('
     my $code = '
 sub P6C::'.$rulename.'::tree {
     my $x = shift;
@@ -642,8 +876,8 @@ sub P6C::'.$rulename.'::tree {
 	    # No required, multiple optional.
 	    $rule .= argtype($params->opt(0)->var->type);
 	    $rule .= " (',' scalar_expr)(0..".(@{$params->opt} - 1).')';
-	    $rule .= '
-		| # nothing';
+	    $rule .= "
+		| ...!'(' # nothing";
 	    $code .= '
     die "Internal error" unless @$x == 1 || @$x == 3;
     if (@$x == 3) {
