@@ -9,6 +9,7 @@ use vars qw(@steps);
 	init/data.pl
 	init/miniparrot.pl
 	init/hints.pl
+	init/debug.pl
 	inter/progs.pl
 	inter/types.pl
 	inter/ops.pl
@@ -17,6 +18,7 @@ use vars qw(@steps);
     auto/alignptrs.pl
 	auto/headers.pl
 	auto/sizes.pl
+	auto/stackdir.pl
 	auto/byteorder.pl
 	auto/pack.pl
 	auto/format.pl
@@ -28,6 +30,7 @@ use vars qw(@steps);
 	gen/config_pm.pl
 	gen/makefiles.pl
 	gen/platform.pl
+	gen/libparrot_def.pl
 );
 #	auto/alignptrs.pl
 
