@@ -1,7 +1,7 @@
 #
 # C.pm
 #
-# $Id: C.pm,v 1.12 2002/07/12 18:43:47 grunblatt Exp $
+# $Id: C.pm,v 1.13 2002/08/19 23:16:54 tom Exp $
 #
 
 use strict;
@@ -57,13 +57,15 @@ sub expr_pop {
   'n'  => "interpreter->ctx.num_reg.registers[cur_opcode[%ld]]",
   'p'  => "interpreter->ctx.pmc_reg.registers[cur_opcode[%ld]]",
   's'  => "interpreter->ctx.string_reg.registers[cur_opcode[%ld]]",
-  'k'  => "interpreter->ctx.int_reg.registers[cur_opcode[%ld]]",
+  'k'  => "interpreter->ctx.pmc_reg.registers[cur_opcode[%ld]]",
+  'ki'  => "interpreter->ctx.int_reg.registers[cur_opcode[%ld]]",
   
   'ic' => "cur_opcode[%ld]",
   'nc' => "interpreter->code->const_table->constants[cur_opcode[%ld]]->number",
   'pc' => "%ld /* ERROR: Don't know how to handle PMC constants yet! */",
   'sc' => "interpreter->code->const_table->constants[cur_opcode[%ld]]->string",
-  'kc' => "cur_opcode[%ld]"
+  'kc' => "interpreter->code->const_table->constants[cur_opcode[%ld]]->key",
+  'kic' => "cur_opcode[%ld]"
 );
 
 sub access_arg
