@@ -125,13 +125,15 @@ sub access_arg
     'n'  => "(*(FLOATVAL *)cur_opcode[%ld])",
     'p'  => "(*(PMC **)cur_opcode[%ld])",
     's'  => "(*(STRING **)cur_opcode[%ld])",
-    'k'  => "(*(INTVAL *)cur_opcode[%ld])",
+    'k'  => "(*(PMC **)cur_opcode[%ld])",
+    'ki'  => "(*(INTVAL *)cur_opcode[%ld])",
 
     'ic' => "(*(INTVAL *)cur_opcode[%ld])",
     'nc' => "(*(FLOATVAL *)cur_opcode[%ld])",
     'pc' => "%ld /* ERROR: Don't know how to handle PMC constants yet! */",
     'sc' => "(*(STRING **)cur_opcode[%ld])",
-    'kc' => "(*(INTVAL *)cur_opcode[%ld])"
+    'kc' => "(*(PMC **)cur_opcode[%ld])",
+    'kic' => "(*(INTVAL *)cur_opcode[%ld])"
   );
 
   die "Unrecognized type '$type' for num '$num' in opcode @{[$op->full_name]}"
