@@ -1,7 +1,7 @@
 /* misc.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: misc.c,v 1.22 2002/06/11 20:11:51 dmlloyd Exp $
+ *     $Id: misc.c,v 1.23 2002/08/21 08:00:10 mongo Exp $
  *  Overview:
  *     Miscellaneous functions
  *  Data Structure and Algorithms:
@@ -495,7 +495,7 @@ Parrot_vsprintf(struct Parrot_Interp *interpreter, char *targ, const char *pat,
     STRING *ret = Parrot_vsprintf_c(interpreter, pat, args);
 /*    string_transcode(interpreter, ret, NULL, NULL, &ret);*/
 
-    memcpy(targ, ret->bufstart, ret->bufused);
+    memcpy(targ, ret->strstart, ret->bufused);
     targ[ret->bufused + 1] = 00;
 }
 
@@ -510,7 +510,7 @@ Parrot_vsnprintf(struct Parrot_Interp *interpreter, char *targ,
         len = ret->bufused;
     }
 
-    memcpy(targ, ret->bufstart, len);
+    memcpy(targ, ret->strstart, len);
     targ[len + 1] = 0;
 }
 
