@@ -15,22 +15,26 @@ PXS_initargs(Parrot_Interp_t interp) {
 
 void
 PXS_reti(Parrot_Interp_t interp, INTVAL i) {
-    stack_push(interp, &interp->user_stack, &i, STACK_ENTRY_INT, STACK_CLEANUP_NULL);
+    stack_push(interp, &interp->user_stack, &i, STACK_ENTRY_INT,
+               STACK_CLEANUP_NULL);
 }
 
 void 
 PXS_retn(Parrot_Interp_t interp, FLOATVAL f) {
-    stack_push(interp, &interp->user_stack, &f, STACK_ENTRY_FLOAT, STACK_CLEANUP_NULL);
+    stack_push(interp, &interp->user_stack, &f, STACK_ENTRY_FLOAT,
+               STACK_CLEANUP_NULL);
 }
 
 void
 PXS_rets(Parrot_Interp_t interp, STRING * s) {
-    stack_push(interp, &interp->user_stack, s, STACK_ENTRY_STRING, STACK_CLEANUP_NULL);
+    stack_push(interp, &interp->user_stack, s, STACK_ENTRY_STRING,
+               STACK_CLEANUP_NULL);
 }
 
 void
 PXS_retp(Parrot_Interp_t interp, PMC * p) {
-    stack_push(interp, &interp->user_stack, p, STACK_ENTRY_PMC, STACK_CLEANUP_NULL);
+    stack_push(interp, &interp->user_stack, p, STACK_ENTRY_PMC, 
+               STACK_CLEANUP_NULL);
 }
 
 INTVAL
