@@ -1,7 +1,7 @@
 /* rx.c
  *  Copyright: (When this is determined...it will go here)
  *  CVS Info
- *     $Id: rx.c,v 1.16 2002/06/12 22:58:29 dmlloyd Exp $
+ *     $Id: rx.c,v 1.18 2002/08/22 20:08:23 dan Exp $
  *  Overview:
  *     Supporting file for the regular expression engine
  *  Data Structure and Algorithms:
@@ -27,27 +27,6 @@ const char *RX_NEWLINES = "\r\n";       /* XXX Unicode defines a few more. */
  *               Initial version by Brent Dax                *
  *************************************************************/
 
-rxinfo *
-rx_allocate_info(struct Parrot_Interp *interpreter, STRING *string)
-{
-    rxinfo *rx = mem_sys_allocate(sizeof(rxinfo));
-
-    rx->minlength = rx->index = rx->startindex = 0;
-    rx->flags = enum_rxflags_none;
-    rx->whichway = enum_rxdirection_forwards;
-
-    rx->string = string;
-
-    rx->groupstart = pmc_new(interpreter, enum_class_PerlArray);
-    rx->groupend = pmc_new(interpreter, enum_class_PerlArray);
-
-    rx->stack = intstack_new(interpreter);
-
-    string_transcode(interpreter, rx->string, encoding_lookup("utf32"),
-                     rx->string->type, &rx->string);
-
-    return rx;
-}
 
 INTVAL
 rx_is_word_character(struct Parrot_Interp *interpreter, INTVAL ch)
